@@ -1,6 +1,5 @@
 import pandas as pd
-import numpy as np
-from pathlib import Path, PurePath
+from pathlib import Path
 from io import StringIO
 import re
 import logging
@@ -605,6 +604,8 @@ Pre-processing pipeline
                 edges='open',
                 colLoc='right',
                 )
+            matplotlib_table.auto_set_font_size(False)
+            matplotlib_table.set_fontsize(12)
             if page == 0:
                 self.plt.title(add_title, y=1.1) #pad=20) # -- placement is off
             self.pdf.savefig(fig)
