@@ -53,7 +53,8 @@ def mean_beta_plot(df, verbose=False, save=False, silent=False):
     data = df.copy(deep=True)
     data['mean'] = data.mean(numeric_only=True, axis=1)
     fig, ax = plt.subplots(figsize=(12, 9))
-    sns.distplot(data['mean'], hist=False, rug=False, ax=ax, axlabel='beta')
+    #sns.distplot(data['mean'], hist=False, rug=False, ax=ax, axlabel='beta')
+    sns.kdeplot(data['mean'], ax=ax, label='beta')
     plt.title('Mean Beta Plot')
     plt.grid()
     plt.xlim(0,1.0)
@@ -67,7 +68,7 @@ def mean_beta_plot(df, verbose=False, save=False, silent=False):
 
 
 
-def beta_density_plot(df, verbose=False, save=False, silent=False, reduce=0.1, plot_title=None, ymax=None, return_fig=False, full_range=False, highlight_samples=None, figsize=(12,9), show_labels=None):
+def beta_density_plot(df, verbose=False, save=False, silent=False, reduce=0.1, plot_title=None, ymax=None, return_fig=False, full_range=False, highlight_samples=None, figsize=(12,9), show_labels=None, filename='beta.png'):
     """Returns a plot of beta values for each sample in a batch of samples as a separate line.
     Y-axis values is the count (of what? intensity? normalized?).
     X-axis values are beta values (0 to 1) for a single samples
@@ -92,8 +93,8 @@ def beta_density_plot(df, verbose=False, save=False, silent=False, reduce=0.1, p
             when working with datasets and you don't need publication quality "exact" plots,
             supply a float between 0 and 1 to sample the probe data for plotting.
             We recommend 0.1, which plots 10% of the 450k or 860k probes, and doesn't distort
-            the distribution. Values below 0.001 (860 probes out of 860k) will show some sampling distortion.
-            Using 0.1 will speed up plotting 10-fold.
+            the distribution much. Values below 0.001 (860 probes out of 860k) will show some sampling distortion.
+            Using 0.1 will speed up plotting 10-fold of large batches.
         ymax (None): If defined, upper limit of plot will not exceed this value. But it y-range can be smaller if values are less than this range.
         full_range: (False) if True, x-axis will be auto-scaled, instead of fixed in the 0-to-1.0 range.
         return_fig: (False) if True, returns figure object instead of showing plot.
@@ -124,7 +125,8 @@ def beta_density_plot(df, verbose=False, save=False, silent=False, reduce=0.1, p
     # 2nd check: incomplete probes
     if df.shape[0] < 27000:
         LOGGER.warning("Data does not appear to be full probe data")
-    # 3rd check: missing probe values (common with EPIC+)
+    # 3rd check: missing probe values (common with EPIC+ and pvalues)
+    pre_probe_count = df.shape[0]
     missing_probes = sum(df.isna().sum())
     if missing_probes > 0 and silent:
         df = df.copy().dropna()
@@ -135,6 +137,9 @@ def beta_density_plot(df, verbose=False, save=False, silent=False, reduce=0.1, p
         LOGGER.warning(f"Your data contains {int(missing_probes/len(df.columns))} missing probe values per sample, ({missing_probes} overall).")
         LOGGER.info(df.isna().sum())
         df = df.copy().dropna()
+    if df.shape[0] < 2:
+        print(f"ERROR: Could not plot these samples because there are missing values for each probe in at least one sample in the batch: Started with {pre_probe_count} probes and ended with {df.shape[0]} probes shared after filtering.")
+        return
 
     if reduce != None and reduce < 1.0:
         if not isinstance(reduce, (int, float)):
@@ -152,30 +157,44 @@ def beta_density_plot(df, verbose=False, save=False, silent=False, reduce=0.1, p
     if show_labels is None: # user-param-always overrides this.
         show_labels = True if len(df.columns) <= 30 else False
     for col in df.columns: # samples
-        if col != 'Name': # probe name
-            if reduce:
+        if col not in ('Name','ID_REF'): # probe name
+            if reduce == 1.0:
+                values = df[col].values
+            elif reduce != None and 0 < reduce < 1.0:
                 values = df[col].values[probes]
             else:
                 values = df[col].values
             if len(values.shape) > 1:
                 raise ValueError("Your df probaby contains duplicate sample names.")
 
+            # updated for seaborn v0.11 (kwargs differ from v0.10; used kde_kws["lw"] instead of "linewidth")
             kde_kws = None # optional, conditional color highlighting for one or several samples in the DF, using the "highlight_samples" list passed in.
             if highlight_samples not in (None,[]):
-                kde_kws={"color": "lightgray", "lw": 1}
+                kde_kws={"color": "lightgray", "linewidth": 1}
                 if isinstance(highlight_samples, str) and col == highlight_samples:
-                    kde_kws.update({"color":"b", "lw":3})
+                    kde_kws.update({"color":"b", "linewidth":3}) #"fill": True
                 if isinstance(highlight_samples, list) and col in highlight_samples:
-                    kde_kws.update({"color":"b", "lw":3})
+                    kde_kws.update({"color":"b", "linewidth":3}) #"fill":True
 
-            kwargs = {"hist":False, "rug":False, "kde":True, "ax":ax, "axlabel":'beta'}
+            #kwargs = {"hist":False, "rug":False, "kde":True, "ax":ax, "axlabel":'beta'} seaborn v.10
+            kwargs = {"ax": ax}
             if kde_kws:
-                kwargs["kde_kws"] = kde_kws
+                #kwargs["kde_kws"] = kde_kws seaborn v0.10
+                kwargs.update(kde_kws) # seaborn v0.11
             if show_labels:
                 kwargs["label"] = col
-            sns.distplot(values, **kwargs)
+
+            if values.shape[0] < 2:
+                if verbose:
+                    print(f"Skipping blank sample {values}")
+                continue
+            #sns.distplot(values, **kwargs)
+            sns.kdeplot(values, **kwargs)
 
     (obs_ymin, obs_ymax) = ax.get_ylim()
+    #if verbose: #DEBUG for if plot is blank
+    #    print(values.shape, kwargs, obs_ymin, obs_ymax)
+
     if ymax is not None and obs_ymax > ymax:
         ax.set_ylim(0, ymax)
     if show_labels:
@@ -189,7 +208,7 @@ def beta_density_plot(df, verbose=False, save=False, silent=False, reduce=0.1, p
         plt.xlim(0,1.0)
     plt.xlabel('Beta values')
     if save:
-        plt.savefig('beta.png')
+        plt.savefig(filename)
     if return_fig:
         return fig
     if not silent:
@@ -246,7 +265,8 @@ def cumulative_sum_beta_distribution(df, cutoff=0.7, verbose=False, save=False,
         cumulative_sum = np.cumsum(hist_vals)
         if cumulative_sum[5] < cutoff:
             good_samples.append(subject_num)
-            sns.distplot(row, hist=False, norm_hist=False)
+            #sns.distplot(row, hist=False, norm_hist=False)
+            sns.kdeplot(row)
         else:
             outliers.append(subject_id) # drop uses ids, not numbers.
 
@@ -579,7 +599,12 @@ def beta_mds_plot(df, filter_stdev=1.5, verbose=False, save=False, silent=False,
 
 def mean_beta_compare(df1, df2, save=False, verbose=False, silent=False):
     """Use this function to compare two dataframes, pre-vs-post filtering and removal of outliers.
+args:
+    the first argument (df1) is the "pre" dataframe of samples
+    the second argument (df2) is the "post" dataframe of samples
 
+kwargs:
+    verbose: additional output
     silent: suppresses figure, so no output unless save==True too."""
     if df1.shape[0] < df1.shape[1]:
         ## ensure probes in rows and samples in cols
@@ -600,8 +625,10 @@ def mean_beta_compare(df1, df2, save=False, verbose=False, silent=False):
     data2['mean'] = data2.mean(numeric_only=True, axis=1)
 
     fig, ax = plt.subplots(figsize=(12, 9))
-    line1 = sns.distplot(data1['mean'], hist=False, rug=False, ax=ax, axlabel='beta', color='xkcd:blue')
-    line2 = sns.distplot(data2['mean'], hist=False, rug=False, color='xkcd:green')
+    #line1 = sns.distplot(data1['mean'], hist=False, rug=False, ax=ax, axlabel='beta', color='xkcd:blue')
+    #line2 = sns.distplot(data2['mean'], hist=False, rug=False, color='xkcd:green')
+    line1 = sns.kdeplot(data1['mean'], ax=ax, label='beta', color='xkcd:blue')
+    line2 = sns.kdeplot(data2['mean'], color='xkcd:green')
     plt.title('Mean Beta Plot (Compare pre (blue) vs post (green) filtering)')
     plt.grid()
     plt.xlabel('Mean Beta')
